@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.7 2003/07/05 06:32:33 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.9 2003/07/20 14:44:37 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -39,18 +39,13 @@
 #include <fcntl.h>
 #endif
 
-#include "server/postgres_fe.h"
-#include "server/libpq/pqcomm.h"
-
 #include "pool.h"
 
 #ifdef NONE_BLOCK
 static void set_nonblock(int fd);
 #endif
-
-#ifdef RESET_NONBLOCK
 static void unset_nonblock(int fd);
-#endif
+
 static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd);
 static StartupPacket *read_startup_packet(POOL_CONNECTION *cp);
 static int send_startup_packet(POOL_CONNECTION *cp, StartupPacket *sp);
@@ -68,13 +63,14 @@ void do_child(int unix_fd, int inet_fd)
 	pool_debug("I am %d", getpid());
 
 	/* set up signal handlers */
+	signal(SIGALRM, SIG_DFL);
 	signal(SIGTERM, SIG_DFL);
 	signal(SIGINT, SIG_DFL);
 	signal(SIGCHLD, SIG_DFL);
 	signal(SIGUSR1, SIG_DFL);
 
 #ifdef NONE_BLOCK
-	/* set fds to none block */
+	/* set listen fds to none block */
 	set_nonblock(unix_fd);
 	if (inet_fd)
 	{
@@ -90,7 +86,7 @@ void do_child(int unix_fd, int inet_fd)
 
 	for (;;)
 	{
-		bool connection_reuse = true;
+		int connection_reuse = 1;
 
 		/* perform accept() */
 		frontend = do_accept(unix_fd, inet_fd);
@@ -100,14 +96,8 @@ void do_child(int unix_fd, int inet_fd)
 			continue;
 		}
 
-#ifdef RESET_NONBLOCK
-		/* unset none block */
-		unset_nonblock(unix_fd);
-		if (inet_fd)
-		{
-			unset_nonblock(inet_fd);
-		}
-#endif
+		/* unset frontend fd tp none block */
+		unset_nonblock(frontend->fd);
 
 		/* read the startup packet */
 		sp = read_startup_packet(frontend);
@@ -125,17 +115,20 @@ void do_child(int unix_fd, int inet_fd)
 		 */
 		if ((backend = pool_get_cp(sp->user, sp->database)) == NULL)
 		{
-			connection_reuse = false;
+			connection_reuse = 0;
 
 			/* connect to the backend and send the startup packect here */
 			backend = pool_create_cp(sp->user, sp->database);
 			if (backend == NULL)
 			{
-			  /* pool_get_cp  no more connection ä顤򥯥饤Ȥ֤
-			     ɬפ */
+				static char *msg = "Sorry, too many clients already";
+
+				pool_error("do_child: cannot not create backend connection");
+
+				pool_write(frontend, "E", 1);
+				pool_write_and_flush(frontend, msg, strlen(msg)+1);
 				pool_close(frontend);
-				pool_error("backend connection failed");
-				exit(1);
+				continue;
 			}
 
 			/* mark this is a backend connection */
@@ -144,7 +137,7 @@ void do_child(int unix_fd, int inet_fd)
 			/* send startup packet */
 			if (send_startup_packet(backend->con, sp) < 0)
 			{
-				pool_error("failed to send startup packet to the backend");
+				pool_error("do_child: fails to send startup packet to the backend");
 				pool_close(frontend);
 				continue;
 			}
@@ -193,6 +186,8 @@ void do_child(int unix_fd, int inet_fd)
 						pool_discard_cp(sp->user, sp->database);
 
 					pool_close(frontend);
+
+					pool_connection_pool_timer(backend);
 					break;
 				
 				/* error occured. discard backend connection pool
@@ -250,7 +245,6 @@ static void set_nonblock(int fd)
 }
 #endif
 
-#ifdef RESET_NONBLOCK
 /*
  * unset non-block flag
  */
@@ -271,7 +265,6 @@ static void unset_nonblock(int fd)
 		exit(1);
 	}
 }
-#endif
 
 /*
 * perform accept() and returns new fd
@@ -367,8 +360,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 
 	pool_debug("Protocol Version: %08x",ntohl(sp.protoVersion));
 	pool_debug("Protocol Major: %d Minor:%d",
-		   PG_PROTOCOL_MAJOR(ntohl(sp.protoVersion)),
-		   PG_PROTOCOL_MINOR(ntohl(sp.protoVersion)));
+		   ntohl(sp.protoVersion)>>16,
+			   ntohl(sp.protoVersion) & 0x0000ffff);
 	pool_debug("database: %s",sp.database);
 	pool_debug("user: %s",sp.user);
 
@@ -395,7 +388,13 @@ static void reset_backend(POOL_CONNECTION *backend)
 {
 	char kind;
 	int len;
+
+#ifdef NO_RESET_ALL
+	static char *queries[] = {"ABORT"};
+#else
 	static char *queries[] = {"ABORT", "RESET ALL"};
+#endif
+
 	int i;
 
 	for (i=0;i<sizeof(queries)/sizeof(char *);i++)
