@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_error.c,v 1.1.1.1 2003/06/24 05:23:55 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_error.c,v 1.2 2003/07/20 14:44:37 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -33,7 +33,7 @@ void pool_error(const char *fmt,...)
 {
 	va_list		ap;
 
-	fprintf(stderr, "ERROR: ");
+	fprintf(stderr, "ERROR: pid %d: ", getpid());
 	va_start(ap, fmt);
 	vfprintf(stderr, fmt, ap);
 	va_end(ap);
@@ -47,7 +47,7 @@ void pool_debug(const char *fmt,...)
 	if (!debug)
 		return;
 
-	fprintf(stderr, "DEBUG: ");
+	fprintf(stderr, "DEBUG: pid %d: ", getpid());
 	va_start(ap, fmt);
 	vfprintf(stderr, fmt, ap);
 	va_end(ap);
